#ifndef WINDOW_HPP
#define WINDOW_HPP

#include "model.hpp"
#include "protocols.hpp"

#include <QComboBox>
#include <QGuiApplication>
#include <QLineEdit>
#include <QMainWindow>
#include <QStyledItemDelegate>
#include <QTableView>

#include <functional>
#include <memory>

QT_BEGIN_NAMESPACE
namespace Ui {
class Window;
}
QT_END_NAMESPACE

class Window : public QMainWindow {
    Q_OBJECT
    std::map<ProtocolSource, std::unique_ptr<Model>> models;

  public:
    Window(QNativeInterface::QWaylandApplication* waylandApp, QWidget* parent = nullptr);
    ~Window();

    void newGlobal(const std::string interface, const uint32_t version);

    void addProtocol(const Protocol& protocol);

  private:
    void initTable(const ProtocolSource source, QTableView& table);

    Model* modelForStatus(const ProtocolStatus status);

  private:
    std::unique_ptr<Ui::Window> const ui;
    QLineEdit searchBox;
    QComboBox supportBox;
};

class TableItemDelegate : public QStyledItemDelegate {
  public:
    TableItemDelegate(QObject* parent = nullptr);

    void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const override;
};

#endif
